#!/usr/bin/env python2.1

##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

""" replay server """

__version__ = "$Id: rp_server.py,v 1.3 2002/02/25 18:45:41 andreas Exp $"

# Python stuff first
import os,sys,threading,getopt,Queue
import time,traceback
import ConfigParser
from websiteloadtest.common import timeoutsocket
timeoutsocket.setDefaultSocketTimeout(10)


# prepare Globals module
from websiteloadtest.common import Globals

Globals.CONFIG = ConfigParser.ConfigParser()
Globals.CONFIG.read(['rp_server.cfg',os.path.expanduser('~/.rp_server.cfg')])


# now our app specific modules
from websiteloadtest.server import LogUDPServer,Heartbeat,Registry,ClientReplayFeeder
from websiteloadtest.client import LogUDPClient
from websiteloadtest.common.xmlrpcreply import XRR
from websiteloadtest.common import XmlRPCServer
from websiteloadtest.client import LogUDPClient
from urllib import getproxies


# Proxy handling

http_proxy = getproxies().get('http')
if http_proxy:
    http_proxy = http_proxy.replace('http://','')
    if http_proxy.find(":"):
        PROXY_HOST,PROXY_PORT = http_proxy.split(':')
    else:
        PROXY_HOST,PROXY_PORT = http_proxy,'80'        

else:
    PROXY_HOST = ''
    PROXY_PORT = ''


def readConfig(env):

    import socket

    SERVER_PORT = Globals.CONFIG.get('server','port')
    LOG_SERVER_PORT = Globals.CONFIG.get('logger','port')
    STAT_LOG_SERVER_PORT = Globals.CONFIG.get('statslogger','port')

    h = socket.gethostname()
    if h.find('.')>-1:
        SERVER_HOST = h.split('.')[0]
    else:
        SERVER_HOST = h

    Globals.SERVER_HOST = env['SERVER_HOST'] = SERVER_HOST
    Globals.SERVER_PORT = env['SERVER_PORT'] = int(SERVER_PORT)
    Globals.LOG_SERVER_HOST = env['LOG_SERVER_HOST'] = SERVER_HOST
    Globals.LOG_SERVER_PORT = env['LOG_SERVER_PORT'] = int(LOG_SERVER_PORT)
    Globals.STAT_LOG_SERVER_HOST = env['STAT_LOG_SERVER_HOST'] = SERVER_HOST
    Globals.STAT_LOG_SERVER_PORT = env['STAT_LOG_SERVER_PORT'] = int(STAT_LOG_SERVER_PORT)

    Globals.CLIENT_HOST = SERVER_HOST


class handlerReplayServer(XmlRPCServer.xmlrpcHandler):

    def call(self, method, args):
        """ callback for requests """

        # Check if we have something like 'class.method'

        if method.find('.')>-1:
            cls,meth = method.split('.')

            # Try to get instance

            
            try:

                inst = getattr(Globals,cls,None)
                res = apply(getattr(inst,meth),args)
                return res

            except:
                traceback.print_exc()
                raise RuntimeError,"No such method '%s'" % method

        else:
            return XmlRPCServer.xmlrpcHandler.call(self,method,args)


    def heartbeat(self):
        return XRR(1,time.time())


    def uptime(self):
        import popen2

        sout,sin = popen2.popen2('uptime')
        uptime = sout.read()

        return XRR(1,uptime)


    def getConfig(self):

        d = {}
        for k in ['SERVER_HOST','SERVER_PORT','LOG_SERVER_HOST',\
                  'LOG_SERVER_PORT', 'STAT_LOG_SERVER_PORT', \
                  'STAT_LOG_SERVER_HOST','PROXY_HOST','PROXY_PORT' ]:
            v = globals()[k]

            d[k] = v

        return XRR(1,d)


    def getData(self):
        """ return some replay data for client """

        # client should not request data when the feeder
        # is not running

        if Globals.REPLAY_FEEDER.isSuspended():
            print 'getdata: feeder suspended'
            return XRR(0,'Feeder suspended')

        # Feeder is running but Queue is empty 
        if Globals.REPLAY_FEEDER.empty():   
            return XRR(0,'Queue empty')

        try:

            data = Globals.REPLAY_FEEDER.get_nowait()
            return XRR(1,data)

        except Queue.Empty:
            return XRR(0,'Queue empty')

    def test(self):
        return 'a',2,(2,3)


    def replayer_queueSize(self):
        return XRR(1,Globals.REPLAY_FEEDER.qsize())


    def replayer_queueClear(self):

        try:
            Globals.REPLAY_FEEDER.clear()
            return XRR(1,'Done')
        except:
            return XRR(0)


    def replayer_start_feeding(self):
        """ start a thread to feed the queue """
        
        if Globals.REPLAY_FEEDER.isSuspended():

            Globals.REPLAY_FEEDER.setQueues(len(Globals.REGISTRY))
            Globals.REPLAY_FEEDER.resume()
            Globals.LOG.log_info('feeding replayer clients resumed')

            return XRR(1,'Done')

        else:
       
            return XRR(0,"Feeding already in progress")


    def replayer_feed_file(self,fname):
        """ instruct replay feeder to queue a file """

        if Globals.REPLAY_FEEDER.isSuspended():

            if Globals.REPLAY_FEEDER.feedFile(fname):
                Globals.LOG.log_info('feed file set to',fname)
                return XRR(1,'Done')
            else:
                Globals.LOG.log_info('feeding failed for ',fname)
                return XRR(0,'Feeding failed - please check logs')

        else:
            return XRR(0,"Replay feeder is running")


    def replayer_stop_feeding(self):
        """ stop the  thread to feed the queue """
        
        if not Globals.REPLAY_FEEDER.isSuspended():
 
            Globals.REPLAY_FEEDER.suspend()
            Globals.LOG.log_info('feeding replayer clients stopped')

            return XRR(1,'Done')            

        else:
            return XRR(0,"Feeding is already suspended")



class ReplayServer(XmlRPCServer.XmlRPCServer):
    """ replay server """

    def __init__(self, port):

        XmlRPCServer.XmlRPCServer.__init__( 
                self, 
                port, handlerReplayServer 
                )
        print "ReplayServer started on port",port 
            


def init():

    # create registry for clients
    Globals.REGISTRY = Registry.Registry()

    # start thread with XMLRPC server
    Globals.REPLAY_XML_SRV= ReplayServer(SERVER_PORT)
    Globals.REPLAY_XML_SRV.start()

    # start UDP logger thread
    Globals.LOGGER = LogUDPServer.LogUDPServer(
            LOG_SERVER_PORT,
            Globals.CONFIG.get('logger','logfile')
            )
    Globals.LOGGER.start()

    # start UDP logger thread to collect statistics from clients
    Globals.STATS_LOGGER = LogUDPServer.LogUDPServer(
            STAT_LOG_SERVER_PORT,
            Globals.CONFIG.get('statslogger','logfile')
            )

    Globals.STATS_LOGGER.start()

    # start thread to check clients
    Globals.HEARTBEAT = Heartbeat.Heartbeat(registry=Globals.REGISTRY)
    Globals.HEARTBEAT.start()

    # start client replay feeder thread
    Globals.REPLAY_FEEDER = ClientReplayFeeder.ClientReplayFeeder()
    Globals.REPLAY_FEEDER.start() 

    Globals.LOG = LogUDPClient.Logger(
                LOG_SERVER_HOST,LOG_SERVER_PORT)


def shutdown():

    for x in threading.enumerate():

        if x != threading.currentThread():
            print "Trying to shutdown thread",x
            x.stop()
            x.join()


def usage():
    print "%s [options] <>" % os.path.basename(sys.argv[0])
    print
    print "Options:"
    print "    None"
    print
    print "Version:",__version__



if __name__=='__main__':

    readConfig( globals() )

    opts,args = getopt.getopt(sys.argv[1:],'hp:',
            ['help','port='])

    for k,v in opts:
        if k in ['--help','-h']:        usage(); sys.exit(1) 
        if k in ['--port','-p']:        SERVER_PORT = int(v)

    init()

    while 1:
        try:

            time.sleep(0.2)

        except: 
            traceback.print_exc()
            shutdown()            
            os._exit(0)
