#!/usr/bin/env python2.1

##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

""" replay server """

__version__ = "$Id: rpsh.py,v 1.4 2002/02/26 14:56:21 andreas Exp $"

import os,sys,getopt,atexit,traceback
import ConfigParser

from websiteloadtest.common import timeoutsocket
timeoutsocket.setDefaultSocketTimeout(2)
from websiteloadtest.xmlrpclib import xmlrpclib

CONFIG = ConfigParser.ConfigParser()
CONFIG.read(['rp_client.cfg',os.path.expanduser('~/.rp_client.cfg')])

SERVER_PORT = CONFIG.getint('server','port')
SERVER_HOST = CONFIG.get('server','host')

try:
    import readline
    histfile = os.path.expanduser('~/.pyhist')
    readline.read_history_file(histfile)
    atexit.register(readline.write_history_file,histfile)
except: pass


class Shell:
    """ replayer shell """

    def pr_under(self,s):
        print
        print s
        print '-' * (len(s))
        print


    def XRRcheck(self,xrr):
        if xrr['status'] == 0:
            
            print 'Error: %s' % xrr['msg']
        else:
            print 'Successful: %s' % xrr['msg']

        
    def getClientServer(self,ci):
        """ returns a XMLRPC client connection object
            for a client represented by 'ci'. 'ci' is 
            a dictionary that contains 'host' and 'port'
            as keys
        """

        srv = xmlrpclib.Server('http://%s:%d' % \
                    (ci['host'],ci['port']), verbose=0)

        return srv


    def _allClients(self,host=None):
        
        all_clients = self.rp_server.REGISTRY.allClients()

        if host=='all' or host==None:
            return all_clients

        if  host in all_clients:
            return [host]

        print "Error: no registered host '%s'"  % host
        return []
                    

    def handle_version(self):
        """ print version """
        print __version__


    handle_version.__usage__ = "version"    


    def handle_start(self,host):
        """ start the replay mechanism on one client """

        for client in self._allClients(host):

            ci = self.rp_server.REGISTRY.getClientInfo(client)

            srv = self.getClientServer(ci)
            res = srv.start_replay()

            self.XRRcheck(res)

    handle_start.__usage__ = "stop [host|all]"


    def handle_die(self,host):
        """ kill one client """

        for client in self._allClients(host):

            ci = self.rp_server.REGISTRY.getClientInfo(client)

            srv = self.getClientServer(ci)

            try:
                srv.die()
            except: pass


    handle_die.__usage__ = "die [host|all]"


    def handle_stop (self,host ):
        """ stop the replay mechanism on one client """

        for client in self._allClients(host):

            ci = self.rp_server.REGISTRY.getClientInfo(client)

            srv = self.getClientServer(ci)
            res = srv.stop_replay()

            self.XRRcheck(res)


    handle_stop.__usage__ = "stop [host|all]"


    def handle_rpstart(self):
        """ start the client replay feeder """

        res = self.rp_server.replayer_start_feeding()
        self.XRRcheck(res)


    def handle_rpstop(self):
        """ stop the client replay feeder """

        res = self.rp_server.replayer_stop_feeding()
        self.XRRcheck(res)

    def handle_test(self):
        """ stop the client replay feeder """

        res = self.rp_server.test()
        print res

    def handle_rpqueue(self, file):
        """ specify the file to be queued to clients """

        res = self.rp_server.replayer_feed_file(file)
        self.XRRcheck(res)

    handle_rpqueue.__usage__ = "rpqueue <filename>"


    def handle_clients(self,*args,**kw):
        """return list of all registered clients """

        self.pr_under("Client info")

        for client in self._allClients():

            # XMLRPC returns the instance of Client as
            # dictionary of its instances variables

            ci = self.rp_server.REGISTRY.getClientInfo(client)
            if str(ci['active'])=='1':
                status_rp = self.getClientServer(ci).get_replayer_status()
            else:
                status_rp = 'n/a'

            print '%-16s:  %s:%d, active=%d,replayer: %s ' % \
                (client,ci['host'],ci['port'],ci['active'],status_rp)

        print                 

    handle_clients.__usage__ = 'clients [host1],[host2],...'


    def handle_uptime(self,host=None):
        """return the uptime and load of the server """
    
        self.pr_under("Uptime")

        if host:
    
            for client in self._allClients(host):
                ci = self.rp_server.REGISTRY.getClientInfo(client)
                srv = self.getClientServer(ci)
            
                res = srv.uptime()

                print client
                self.XRRcheck(res)
            
        else:
            res = self.rp_server.uptime()
            self.XRRcheck(res)            


    handle_uptime.__usage__ = 'uptime [host|all]'


    def handle_status(self,*args,**kw):
        """check if server is alive """
    
        self.pr_under("Status")

        try:
            self.rp_server.heartbeat()
            print "Server is alive"
        except:
            print "Server is dead"

        res = self.rp_server.replayer_queueSize()
        print "Queue length: ",res['msg']

        res = self.rp_server.getConfig()
        print "Config: ",res['msg']


    handle_status.__usage__ = "status <no args>"        


    def handle_exit(self,*args,**kw):
        """terminate program"""
    
        print "goodbye master....."
        os._exit(0)


    handle_exit.__usage__ = "exit"        
    handle_quit = handle_exit
        

    def handle_help(self,cmd=''):
        """display help"""
        self.pr_under("Help")

        print 'For help on a command, type "help <command>"'
        print

        if cmd:
            
            m =  getattr(self,'handle_'+cmd,None)
            if m:
                print "Synopsis:"
                print getattr(m,'__doc__','no help available')
                print
                print "Usage:"
                print getattr(m,'__usage__','no usage available')
            else:
                print 'No such command: %s' % cmd

        else:
            print "Available commands:"
            print

            funcs = filter(lambda x: x.startswith('handle_'),dir(self.__class__))
            funcs.sort()

            for attr in funcs:
                print '%-15s -- %s' % (attr[7:],getattr(getattr(self,attr),'__doc__').strip())

    handle_help.__usage__ = 'help <command>'


    def __init__(self):
        self.rp_server = \
            xmlrpclib.Server("http://%s:%d" % (SERVER_HOST,SERVER_PORT), verbose=0)


    def run(self):

        print "Welcome to the replayer shell"
        print __version__
        print 
        print "For help type 'help'"

        while 1:
            l = raw_input("rpsh> ")
            self.parse_input(l)


    def parse_input(self,line):
        if len(line)==0: return

        f = line.split(' ')
        f = filter(lambda x: x!='', f)

        m =  getattr(self,'handle_'+f[0],None)

        if m is None:
            print 'Unknown command: %s' % f[0]
            return 

        if len(f) > m.func_code.co_argcount:
            print 'too much arguments'
            return

        try:            
            res = apply(m,f[1:],{})
            return res
        except:
            traceback.print_exc()
            print "Error executing command"


def usage():
    pass
                

if __name__=='__main__':

    opts,args = getopt.getopt(sys.argv[1:],'h',
            ['help'])

    for k,v in opts:
        if k in ['--help','-h']:        usage(); sys.exit(1) 

    shell = Shell()

    try:
        shell.run()
    except (KeyboardInterrupt,EOFError):
        pass
        
    print "\ngood bye"
