#!/usr/bin/env python2.1

import os,sys,threading,getopt
import time,traceback,socket
import myhttplib
import Queue
from Request import Request
from LogWriter import LogWriter
from urllib import getproxies

__version__ = "$Id: replay.py,v 1.1.1.1 2001/12/21 20:53:40 andreas Exp $"

# Logfile
defaultLogfile      = 'replay.log'

# debug level
debuglevel          = 0

# maximum threads allowed to idle
max_idle_threads    = 10

# maximum number of threads to kill in one run
max_threads_to_kill = 2


# don't change anything below

IDLE_QUEUE  = Queue.Queue()
LOGWRITER   = LogWriter(defaultLogfile)
I_MUST_DIE  = 'Die now!'


def PostDataFile(fname):
    """ read POST data from a file """

    try:
        return open(fname).read()
    except:
        print 'file not found:',fname
        return ''



class Worker(threading.Thread):
    """ our worker thread class """

    def __init__(self, id):

        threading.Thread.__init__(self)

        self.start_time = time.time()
        self.queue      = Queue.Queue()
        self.running    = 1
        self.id         = id


    def queue_request(self,req):
        self.queue.put(req)


    def threadTime(self):
        return time.time() - self.start_time        

    def run(self):
        
        req = None
    
        while self.running == 1:

            req = self.queue.get()

            if req == I_MUST_DIE:
                self.running = 0 
                return

            try:
                start_ts = time.time()

                status,msg,headers,data = req.send()
                
                duration = time.time() - start_ts

                LOGWRITER('#%02d/%5.1f sec, Dur: %7.4f, %03d %s' % \
                (self.id,self.threadTime(),duration,status,req.str_short()))
            except:

#                traceback.print_exc(file=sys.stdout)
                LOGWRITER('#%02d/%5.1f sec, Dur: %7.4f,  ERR %s' % \
                (self.id,self.threadTime(),0.0,req.str_short()))

            IDLE_QUEUE.put(self)


class WorkerFactory:

    def __init__(self):
        self._workers = {}
        self._num     = 0 

    def newWorker(self):
        """ create a new worker """

        w = Worker(self._num)
        self._workers[self._num] = w
        w.start()

        self._num+=1

        return w 


    def delWorker(self,id):
        """ terminate and kill a worker thread """
        
        th = self._workers[id]
    
        th.queue_request(I_MUST_DIE)
        th.join()

        del self._workers[id]

    def terminateAll(self):

        for num in self._workers.keys():
            self.delWorker(num)


    def __len__(self):
        return len(self._workers)        



def adjust_workers(WF):
    """ Ajust number of worker threads """

    num = 0

    while  IDLE_QUEUE.qsize() > max_idle_threads:

        worker = IDLE_QUEUE.get()
        WF.delWorker(worker.id)

        num+=1
        if num>max_threads_to_kill: return


def replay_file(fname):

    fp = open(fname)

    num = 0


    WF = WorkerFactory()
    
    for line in fp.xreadlines():

        num+=1

        try:
            req = eval(line[:-1])
        except: 
            print sys.exc_type,sys.exc_value
            continue

        num_all_threads = len(WF)

        while req.timestamp > time.time()-start_ts:
            """ let us idle if we have some time for the next request """

            time.sleep(0.5)
            adjust_workers(WF)

            print "%5.1f: " % (time.time() - start_ts),
            print ' Threads: idle=%d, total=%d' % \
                (num_all_threads - IDLE_QUEUE.qsize(),num_all_threads),
            print ' Requests: sent=%d' % num


        if IDLE_QUEUE.empty():
            """ factory for new worker threads """

            worker = WF.newWorker()
            IDLE_QUEUE.put(worker)


        th = IDLE_QUEUE.get_nowait()
        th.queue_request(req)

        adjust_workers(WF)

    WF.terminateAll()    


def usage():
    print "%s [options] <replay.dat>" % os.path.basename(sys.argv[0])
    print
    print "Options:"
    print "    None"
    print
    print "Version:",__version__



if __name__=='__main__':

    opts,args = getopt.getopt(sys.argv[1:],'h',
            ['help'])

    for k,v in opts:
        if k in ['--help','-h']:        usage(); sys.exit(1) 


    fname = args[0]

    try:
        
        print "Start replaying from '%s'" % fname 
        print "Using proxy:",getproxies().get('http',None)
        print

        start_ts = time.time()
        replay_file(fname)

        print "\n%5.2f seconds elapsed" % (time.time() - start_ts)

    except KeyboardInterrupt:
        os._exit(1)

    except:
        print sys.exc_type,sys.exc_value
        os._exit(1)
