##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
# 
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
# 
##############################################################################

import threading,time
import traceback
from websiteloadtest.xmlrpclib import xmlrpclib
from websiteloadtest.common import Globals
from websiteloadtest.common.xmlrpcreply import XRR


class Heartbeat(threading.Thread):
    """ heartbeat thread"""

    running = 1

    def __init__(self, registry) :
        threading.Thread.__init__(self)
        self.setName("SrvHeartbeat")

        self.interval = Globals.CONFIG.getint('heartbeat','rate')
        self.registry = registry

        print 'Heartbeat started, interval=%d seconds' % \
                    self.interval


    def run(self):

        while self.running:

            clients = self.registry.allClients()

            for host in clients:

                client = self.registry.clients[host]

                try:
                    srv = xmlrpclib.Server('http://%s:%s' % \
                        (client.host,client.port))

                    Globals.LOG.log_debug("sending heartbeat request to %s" % host)
                    res = srv.heartbeat()

                    if res['status']>0:
                        client_ts = float(res['msg'])
 
                    ts_diff = abs(client_ts - time.time())
                    if ts_diff > 5:
                        Globals.LOG.log_warn('Warning: timedrift for %s: %s secs' % \
                            (host,ts_diff),p=1)
                       
                    self.registry[host].setActive()
                    Globals.LOG.log_debug("%s is up" % host)

                except :
                    traceback.print_exc()
                    Globals.LOG.log_warn("'%s' is not responding to heartbeat request" % host)

                    tries = self.registry[host].incTries()
                    self.registry[host].setInactive()

                    if tries > 2:
                        self.registry.unregister_client(host)
                        Globals.LOG.log_error("unregister '%s' because of three failed heartbeat reqs." % \
                             host)

            i = 0
            while self.running and i<self.interval*10:
                time.sleep(0.1)
                i+=1

        Globals.LOG.log_info("%s terminated" % self.getName(),p=1)    



    def stop(self):
        # HACK !!!!!
        self.running = 0

